﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using BusinessObjects.DSWS;
using BusinessObjects.DSWS.BIPlatform;
using BusinessObjects.DSWS.BIPlatform.Desktop;
using BusinessObjects.DSWS.Session;


namespace BOEWebServicesTutorial
{
  public partial class BOEWSTutorialForm : Form
  {
    BusinessObjects.DSWS.Connection _boConnection = null;
    Session _boSession = null;
    BIPlatform _boPService = null;
    private static string WS_URL_SESSION = "Session";
    private static string WS_URL_REPORTENGINE = "reportengine";
    private static string WS_URL_BIPLATFORM = "biplatform";

    public BOEWSTutorialForm()
    {
      InitializeComponent();
    }


    private void BOEWSTutorialForm_FormClosing(object sender, FormClosingEventArgs e)
    {
      _boSession.Logout();
    }

    private void showError(DSWSException ex)
    {
      MessageBox.Show(ex.Message + "\n\nTarget Site: " + ex.TargetSite + "\n\n Cause Message: " + ex.CauseMessage +
        "\n\nCause Detail: " + ex.CauseDetail);
    }

    private void btnLogin_Click(object sender, EventArgs e)
    {
      if ((txtConnection.Text != string.Empty) && (txtUser.Text != string.Empty) && 
        (txtPW.Text != string.Empty) && (txtCMS.Text != string.Empty))
      {
        bool bLoggedIn = false;
        try
        {
          Cursor = Cursors.WaitCursor;
          lblCMSStatus.Text = "Logging in...";
          lblCMSStatus.Refresh();
          //Set the base URL's based on the information in txtConnection
          WS_URL_SESSION = txtConnection.Text + WS_URL_SESSION;
          WS_URL_REPORTENGINE = txtConnection.Text + WS_URL_REPORTENGINE;
          WS_URL_BIPLATFORM = txtConnection.Text + WS_URL_BIPLATFORM;
          //Create the connection
          _boConnection = new BusinessObjects.DSWS.Connection(WS_URL_SESSION);
          //Create the credentials for logging in
          EnterpriseCredential boCredential = new EnterpriseCredential();
          boCredential.Login = txtUser.Text;
          boCredential.Password = txtPW.Text;
          boCredential.Domain = txtCMS.Text;
          boCredential.AuthType = "secEnterprise";
          //Create the session
          _boSession = new Session(_boConnection);
          //Log in
          _boSession.Login(boCredential);
          //Reset the URL on the connection so that we can get the BI Platform
          _boConnection.URL = WS_URL_BIPLATFORM;
          _boPService = new BIPlatform(_boConnection, _boSession.ConnectionState);
          bLoggedIn = true;
        }
        //Use DSWSException to get any errors - this gives us more info than the base Exception class.
        catch (DSWSException ex)
        {
          showError(ex);
        }
        finally
        {
          Cursor = Cursors.Default;
          if (bLoggedIn)
          {
            tabsTutorial.Enabled = true;
            lblCMSStatus.Text = "Logged in to " + txtCMS.Text;
            lblCMSStatus.Refresh();
          }
          else
          {
            tabsTutorial.Enabled = false;
            lblCMSStatus.Text = "Unable to log in";
            lblCMSStatus.Refresh();
          }
        }
      }
      else
      {
        MessageBox.Show("Connection, User ID, and Password are REQUIRED!", "Unable to Log In", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
      }
    }

    #region Basics
    private void btnGetID_Click(object sender, EventArgs e)
    {
      try
      {
        //If you're used to the old query language format note the differences here:
        //Add "query://{" at the beginning and "}" at the end.
        string qry = "query://{Select SI_ID, SI_CUID, SI_KIND from CI_INFOOBJECTS where SI_NAME = '" + txtName1.Text + "'}";
        ResponseHolder rh = _boPService.Get(qry, null);
        InfoObjects ioObjs = rh.InfoObjects;
        if (ioObjs == null)
        {
          MessageBox.Show(txtName1.Text + " Not Found", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }
        else
        {
          //If you're used to the old SDK, notice that the index on the InfoObjects collection now starts at 0 instead of 1.
          InfoObject iobj = ((InfoObject)ioObjs.InfoObject[0]);
          txtID1.Text = iobj.ID.ToString();
          txtCUID1.Text = iobj.CUID;
          txtKind1.Text = iobj.Kind;
        }
      }
      catch (DSWSException ex)
      {
        showError(ex);
      }
    }

    private void btnGetName_Click(object sender, EventArgs e)
    {
      try
      {
        //If you're used to the old query language format note the differences here:
        //Add "query://{" at the beginning and "}" at the end.
        string qry = "query://{Select SI_NAME, SI_CUID, SI_KIND from CI_INFOOBJECTS where SI_ID = " + txtID2.Text + "}";
        ResponseHolder rh = _boPService.Get(qry, null);
        InfoObjects ioObjs = rh.InfoObjects;
        if (ioObjs == null)
        {
          MessageBox.Show(txtID2.Text + " Not Found", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }
        else
        {
          InfoObject iobj = ((InfoObject)ioObjs.InfoObject[0]);
          txtName2.Text = iobj.Name;
          txtCUID2.Text = iobj.CUID;
          txtKind2.Text = iobj.Kind;
        }
      }
      catch (DSWSException ex)
      {
        showError(ex);
      }
    }

    private void btnGetInfo_Click(object sender, EventArgs e)
    {
      try
      {
        //If you're used to the old query language format note the differences here:
        //Add "query://{" at the beginning and "}" at the end.
        string qry = "query://{Select SI_NAME, SI_ID, SI_KIND from CI_INFOOBJECTS where SI_CUID = '" + txtCUID3.Text + "'}";
        ResponseHolder rh = _boPService.Get(qry, null);
        InfoObjects ioObjs = rh.InfoObjects;
        if (ioObjs == null)
        {
          MessageBox.Show(txtCUID3.Text + " Not Found", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }
        else
        {
          //If you're used to the old SDK, notice that the index on the InfoObjects collection now starts at 0 instead of 1.
          InfoObject iobj = ((InfoObject)ioObjs.InfoObject[0]);
          txtName3.Text = iobj.Name;
          txtID3.Text = iobj.ID.ToString();
          txtKind3.Text = iobj.Kind;
        }
      }
      catch (DSWSException ex)
      {
        showError(ex);
      }
    }


    private void btnSInfo_Click(object sender, EventArgs e)
    {
      ResponseHolder rh = _boPService.Get("query://{select * from ci_systemobjects where si_kind='connection' and si_failover_available_until=null and si_authen_method!='server-token' order by si_id}", null);
      InfoObjects ioObjs = rh.InfoObjects;
      lbConnect.Items.Clear();
      if (ioObjs != null)
      {
        foreach (InfoObject iobj in ioObjs.InfoObject)
        {
          BusinessObjects.DSWS.BIPlatform.Desktop.Connection conn =
            (BusinessObjects.DSWS.BIPlatform.Desktop.Connection)iobj;

          //sItem =  + " |ID: " + conn.ID.ToString() + " | CUID:" + conn.CUID + " | " + conn.
          lbConnect.Items.Add("Name: " + conn.Name);
          lbConnect.Items.Add("ID: " + conn.ID.ToString());
          lbConnect.Items.Add("CUID: " + conn.CUID);
          lbConnect.Items.Add("Doc Sender CUID: " + conn.DocSenderCUID);
          lbConnect.Items.Add("Owner CUID: " + conn.OwnerCUID);
          lbConnect.Items.Add("Parent CUID: " + conn.ParentCUID);
          lbConnect.Items.Add("User ID: " + conn.UserID);
          lbConnect.Items.Add("____________________________________");
        }
      }
    }
    #endregion Basics

    #region Folders

    private void btnGetFolders_Click(object sender, EventArgs e)
    {
      Cursor = Cursors.WaitCursor;
      lbFolders.BeginUpdate();
      lbFolders.Items.Clear();
      bool bGood = false;
      string qry = "query://{Select SI_NAME, SI_ID, SI_CUID, SI_PARENTCUID, SI_PATH from CI_INFOOBJECTS where SI_KIND = 'Folder' and ";
      try
      {
        if (txtID4.Text != string.Empty)
        {
          qry = qry + "SI_ID=" + txtID4.Text + "}";
          bGood = true;
        }
        else if (txtName4.Text != string.Empty)
        {
          qry = qry + "SI_Name='" + txtName4.Text + "'}";
          bGood = true;
        }
        else if (txtCUID4.Text != string.Empty)
        {
          qry = qry + "SI_CUID='" + txtCUID4.Text + "'}";
          bGood = true;
        }
        else
        {
          MessageBox.Show("ID, Name, or CUID MUST be provided");
        }

        if (bGood)
        {
          ResponseHolder rh = _boPService.Get(qry, null);
          InfoObjects ioObjs = rh.InfoObjects;
          if (ioObjs.InfoObject == null)
          {
            MessageBox.Show("Folder Not Found", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
          }
          else
          {
            int iID;
            string sItem;

            //Basic folder info about this folder
            InfoObject iobj = ioObjs.InfoObject[0];
            iID = iobj.ID;
            string sPath = "Root Folder\\";
            if (((Folder)iobj).Path.PathFolder != null)
            {
              foreach (PathFolder f in ((Folder)iobj).Path.PathFolder)
              {
                sPath = sPath + f.Name + "\\";
              }
              txtFldrPath.Text = sPath + iobj.Name;
            }
            else
            {
              txtFldrPath.Text = GetPathFromID(iID);
            }
            txtFldrName.Text = iobj.Name;

            //Get parent folder name
            qry = "query://{Select SI_NAME from CI_INFOOBJECTS where SI_CUID = '" + iobj.ParentCUID + "'}";
            rh = _boPService.Get(qry, null);
            ioObjs = rh.InfoObjects;
            iobj = ((InfoObject)ioObjs.InfoObject[0]);
            txtFldrParent.Text = iobj.Name;

            //Get Subfolders
            qry = "query://{Select SI_NAME, SI_ID, SI_CUID from CI_INFOOBJECTS where SI_PARENTID = " + iID + " and SI_KIND = 'Folder'}";
            rh = _boPService.Get(qry, null);
            ioObjs = rh.InfoObjects;
            foreach (InfoObject ioObj in ioObjs.InfoObject)
            {
              sItem = ioObj.Name + ": ID=" + ioObj.ID + " CUID=" + ioObj.CUID;
              lbFolders.Items.Add(sItem);
            }
          }
        }
      }
      catch (DSWSException ex)
      {
        showError(ex);
      }
      finally
      {
        lbFolders.EndUpdate();
        Cursor = Cursors.Default;
      }
    }

    private string GetPathFromID(int iID)
    {
      string qry = "query://{Select SI_NAME, SI_PARENTCUID from CI_INFOOBJECTS where SI_ID = " + iID.ToString() + "}";

      string sPath = string.Empty;
      string sCUID = iID.ToString(); //dummy value for first folder
      while (sCUID != string.Empty)
      {
        ResponseHolder rh = _boPService.Get(qry, null);
        InfoObjects ioObjs = rh.InfoObjects;

        if (ioObjs.InfoObject != null)
        {
          sPath = ioObjs.InfoObject[0].Name + "\\" + sPath;
          sCUID = ioObjs.InfoObject[0].ParentCUID;
          qry = "query://{Select SI_NAME, SI_PARENTCUID from CI_INFOOBJECTS where SI_CUID = '" + sCUID + "'}";
        }
        else
        {
          sCUID = string.Empty;
        }
      }
      return sPath;
    }

    #endregion Folders

    #region TreeView
    private void btnGetTree_Click(object sender, EventArgs e)
    {
      Cursor = Cursors.WaitCursor;
      tvFolders.BeginUpdate();
      tvFolders.Nodes.Clear();
      bool bGood = false;
      try
      {
        string qry = "query://{Select SI_NAME, SI_ID, SI_CUID, SI_PARENTCUID, SI_PATH from CI_INFOOBJECTS where SI_KIND = 'Folder' and ";
        if (txtID5.Text != string.Empty)
        {
          qry = qry + "SI_ID=" + txtID5.Text + "}";
          bGood = true;
        }
        else if (txtName5.Text != string.Empty)
        {
          qry = qry + "SI_Name='" + txtName5.Text + "'}";
          bGood = true;
        }
        else if (txtCUID5.Text != string.Empty)
        {
          qry = qry + "SI_CUID='" + txtCUID5.Text + "'}";
          bGood = true;
        }
        else
        {
          MessageBox.Show("ID, Name, or CUID MUST be provided");
        }

        if (bGood)
        {
          ResponseHolder rh = _boPService.Get(qry, null);
          InfoObjects ioObjs = rh.InfoObjects;
          if (ioObjs.InfoObject == null)
          {
            MessageBox.Show("Folder Not Found", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
          }
          else
          {
            int iID;

            //Basic folder info about this folder
            InfoObject iobj = ioObjs.InfoObject[0];
            iID = iobj.ID;
            string sPath = "Root Folder/";
            TreeNode tnBase = new TreeNode("Root Folder", 0, 0);
            TreeNode tnHold;
            TreeNode newNode;
            sPath = getBasePath(sPath, tnBase, iobj);
            tnHold = tnBase.Nodes[0];
            if (sPath != string.Empty)
            {
              qry = "path://" + sPath;
              rh = _boPService.Get(qry, null);
              ioObjs = rh.InfoObjects;
              if (ioObjs.InfoObject != null)
              {
                //get just the folders first
                foreach (InfoObject ioObj in ioObjs.InfoObject)
                {
                  if (ioObj.Kind == "Folder")
                  {
                    newNode = new TreeNode(ioObj.Name, 0, 0);
                    loadFolderNodes(newNode, sPath, ioObj);
                    tnHold.Nodes.Add(newNode);
                  }
                }
                //Now get the reports
                loadRptInfo(tnHold, ioObjs);
                tnBase.Nodes[0] = tnHold;
              }
            }
          }
        }
      }
      catch (Exception ex)
      {
        if (ex is DSWSException)
        {
          showError((DSWSException)ex);
        }
        else
        {
          MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
      }
      finally
      {
        tvFolders.EndUpdate();
        Cursor = Cursors.Default;
      }
    }

    private void loadFolderNodes(TreeNode newNode, string sPath, InfoObject ioObj)
    {
      throw new NotImplementedException();
    }

    private void loadRptInfo(TreeNode tnHold, InfoObjects ioObjs)
    {
      foreach (InfoObject ioObj in ioObjs.InfoObject)
      {
        if (ioObj.Kind == "Crystal")
        {
          tnHold.Nodes.Add(new TreeNode(ioObj.Name, 1, 1));
        }
        else if (ioObj.Kind == "Webi")
        {
          tnHold.Nodes.Add(new TreeNode(ioObj.Name, 2, 2));
        }
        else if (ioObj.Kind == "Dashboard")
        {
          tnHold.Nodes.Add(new TreeNode(ioObj.Name, 3, 3));
        }
        else
        {
          tnHold.Nodes.Add(new TreeNode(ioObj.Name));
        }
      }
    }

    private string getBasePath(string sPath, TreeNode tnBase, InfoObject iobj)
    {
      string result = sPath;
      TreeNode newNode;
      if (((Folder)iobj).Path.PathFolder != null)
      {
        foreach (PathFolder f in ((Folder)iobj).Path.PathFolder)
        {
          result = result + f.Name + "/";
        }
        result = result + iobj.Name;
        newNode = new TreeNode(sPath, 0, 0);
      }
      else
      {
        newNode = new TreeNode(iobj.Name, 0, 0);
      }
      tnBase.Nodes.Add(newNode);
      return result;
    }

    #endregion TreeView
  }
}
